#ifndef __PX_HW_H__
#define __PX_HW_H__

/*
 * Mediaswitch register addresses
 */

extern unsigned int _MsRead(unsigned long offset);
extern void _MsWrite(unsigned long offset,unsigned int data);
extern void _MsSetBits(unsigned long offset,unsigned int data,unsigned int mask);
extern void _MsAndReg(unsigned long offset,unsigned int data);
extern void _MsOrReg(unsigned long offset,unsigned int data);
extern void *_MsGetBase(void);
extern unsigned long _MsGetBusAddr(void);
extern unsigned long _MsGetSize(void);

/*
 * Note that the internals of how the kernel functions work may change
 * in the future, so please don't use the functions with leading
 * underscores, but rather use the macro
 */
#define MsRead(offset) _MsRead(offset)
#define MsWrite(offset, data) _MsWrite(offset, data)
#define MsSetBits( offset, data, mask) _MsSetBits( offset, data, mask)
#define MsAndReg(offset, data) _MsAndReg(offset, data)
#define MsOrReg(offset, data) _MsOrReg(offset, data)
#define MsGetBase() _MsGetBase()
#define MsGetBusAddr() _MsGetBusAddr()
#define MsGetSize() _MsGetSize()

/*
 * Mediaswitch register addresses
 */
#define MSWITCH_BASE MsGetBase()

/* Addresses are offset because read and write will 
 * take offsets as parameters 
 */
#define MS_SYSTEM_REGISTER 0x000
#define MS_CONTROL_REGISTER 0x004
#define MS_CTL_REG 0x004
#define MS_IM_REG 0x008
#define MS_ISR_REG 0x00c

#define MS_TPIM_REG 0x010
#define MS_TPIS_REG 0x014

#define MS_HEIM_REG 0x018
#define MS_HEIS_REG 0x01c

#define MS_STAT_REG 0x030

#define MS_GPIOSEL0_REG 0x040
#define MS_GPIOSEL1_REG 0x044
#define MS_GPIOIM_REG 0x048
#define MS_GPIOIS_REG 0x04c
#define MS_GPIOOB_REG 0x050
#define MS_GPIOOBE_REG 0x054
#define MS_GPIOIB_REG 0x058

#define MS_FAN_CTRL_REG 0x034

#define MS_VBICTRL_REG 0x05c

#define MS_UART1_BASE 0x100
#define MS_UART2_BASE 0x120
#define MS_UART3_BASE 0x140

#define MS_DMA_CTL(chan,reg) (0x200 + (chan) * 0x10 + (reg))

#define CH_IDELL	0
#define CH_IDE		1
#define CH_PARSER	2
#define CH_AUDMIX	3
#define CH_VBI		4
#define CH_ILINK	5
#define CH_IRIN		6
#define CH_IROUT	7

#define REG_NEXT_ADDR	0x00
#define REG_NEXT_COUNT	0x04
#define REG_CUR_ADDR	0x08
#define REG_CUR_COUNT	0x0c


/* I2C Interfaces Registers */
#define MS_I2C0_REG 0x0600
#define MS_I2C1_REG 0x0604

/*
** IDE_ERROR and IDE_FEATURE register located at the same address offset
** IDE_ERROR is accessed when doing pio read only
** IDE_FEATURE is accessed when doing pio write only
** IDE_STATUS and IDE_COMMAND registers located at the same address offset
** IDE_STATUS is accessed when doing pio read only
** IDE_COMMAND is accessed when doing pio write only
** IDE_SECTOR_NUMBER, IDE_CYLINDER_LOW, IDE_CYLINDER_HIGH, IDE_DEVICE_HEAD[3:0]
** can be used for Logical Block addressing (LBA[27:0])
*/
#define MS_IDE_DATA_REG 0x0400
#define MS_IDE_ERROR_REG 0x0404
#define MS_IDE_FEATURE_REG 0x0404
#define MS_IDE_SECTOR_COUNT_REG 0x0408
#define MS_IDE_SECTOR_NUMBER_REG 0x040c
#define MS_IDE_CYLINDER_LOW_REG 0x0410
#define MS_IDE_CYLINDER_HIGH_REG 0x0414
#define MS_IDE_DEVICE_HEAD_REG 0x0418
#define MS_IDE_STATUS_REG 0x041c
#define MS_IDE_COMMAND_REG 0x041c
#define MS_IDE_ALT_STATUS_REG 0x0438
#define MS_IDE_DEV_CONTROL_REG 0x0438
#define MS_IDE0_PIO_DATA_TIMING_REG 0x0500
#define MS_IDE1_PIO_DATA_TIMING_REG 0x0504
#define MS_IDE0_MWDMA_TIMING_REG 0x0508
#define MS_IDE1_MWDMA_TIMING_REG 0x050c
#define MS_IDE_PIO_COMMAND_TIMING_REG 0x0510
#define MS_IDE_ULTRA_DMA_CONTROL_REG 0x0514
#define MS_IDE0_ULTRA_DMA_TIMING_REG 0x0518
#define MS_IDE1_ULTRA_DMA_TIMING_REG 0x051c

#define MS_IDE_SCRAMBLER_0 0x520
#define MS_IDE_SCRAMBLER_1 0x524
#define MS_IDE_SCRAMBLER_2 0x528
#define MS_IDE_SCRAMBLER_3 0x52c


#define MEDIASWITCH_IBASE	56

#define IRQ_MS_IDELDI		(MEDIASWITCH_IBASE+0)	/* IDE Link List DMA IDLE */
#define IRQ_MS_IDEDI		(MEDIASWITCH_IBASE+1)	/* IDE DMA IDLE */
#define IRQ_MS_PDI		(MEDIASWITCH_IBASE+2)	/* PARSER DMA IDLE */
#define IRQ_MS_AMDI		(MEDIASWITCH_IBASE+3)	/* Audio Mix DMA IDLE */
#define IRQ_MS_VDI		(MEDIASWITCH_IBASE+4)	/* VBI DMA IDLE */
#define IRQ_MS_ILDI		(MEDIASWITCH_IBASE+5)	/* ILINK (1394) DMA IDLE */
#define IRQ_MS_IRIDI		(MEDIASWITCH_IBASE+6)	/* IRin DMA IDLE */
#define IRQ_MS_IRODI		(MEDIASWITCH_IBASE+7)	/* IRout DMA IDLE */
#define IRQ_MS_IDELDE		(MEDIASWITCH_IBASE+8)	/* IDE Link List DMA EMPTY */
#define IRQ_MS_IDEDE		(MEDIASWITCH_IBASE+9)	/* IDE DMA EMPTY */
#define IRQ_MS_PDE		(MEDIASWITCH_IBASE+10)	/* PARSER DMA EMPTY */
#define IRQ_MS_AMDE		(MEDIASWITCH_IBASE+11)	/* Audio Mix DMA EMPTY */
#define IRQ_MS_VDE		(MEDIASWITCH_IBASE+12)	/* VBI DMA EMPTY */
#define IRQ_MS_ILDE		(MEDIASWITCH_IBASE+13)	/* ILINK (1394) DMA EMPTY */
#define IRQ_MS_IRIDE		(MEDIASWITCH_IBASE+14)	/* IRin DMA EMPTY */
#define IRQ_MS_IRODE		(MEDIASWITCH_IBASE+15)	/* IRout DMA EMPTY */
#define IRQ_MS_DMAERR		(MEDIASWITCH_IBASE+16)	/* DMA Error */
#define IRQ_MS_IRRD1		(MEDIASWITCH_IBASE+17)	/* IR Receiver Port 1 Data Available */
#define IRQ_MS_IRRD2		(MEDIASWITCH_IBASE+18)	/* IR Receiver Port 2 Data Available */
#define IRQ_MS_IRRD3		(MEDIASWITCH_IBASE+19)	/* IR Receiver Port 3 Data Available */
#define IRQ_MS_IRTC0		(MEDIASWITCH_IBASE+20)	/* IR Transmitter 0 Command Done */
#define IRQ_MS_IRTC1		(MEDIASWITCH_IBASE+21)	/* IR Transmitter 1 Command Done */
#define IRQ_MS_IRTF1		(MEDIASWITCH_IBASE+22)	/* IR Transmitter 1 FIFO Write Threshold */
#define IRQ_MS_UART1		(MEDIASWITCH_IBASE+23)	/* UART 1 */
#define IRQ_MS_UART2		(MEDIASWITCH_IBASE+24)	/* UART 2 */
#define IRQ_MS_UART3		(MEDIASWITCH_IBASE+25)	/* UART 3 */
#define IRQ_MS_I2C0		(MEDIASWITCH_IBASE+26)	/* I2C port 0 ready */
#define IRQ_MS_I2C1		(MEDIASWITCH_IBASE+27)	/* I2C port 1 ready */
#define IRQ_MS_HE		(MEDIASWITCH_IBASE+28)	/* Hardware Error */
#define IRQ_MS_TP		(MEDIASWITCH_IBASE+29)	/* PARSER Interrupt */
#define IRQ_MS_GPIO		(MEDIASWITCH_IBASE+30)	/* GPIOs Interrupt */
#define IRQ_MS_IDE		(MEDIASWITCH_IBASE+31)	/* IDE device Interrupt */

#define MEDIASWITCH_IHBASE	88

#define IRQ_MSE_DFE0		(MEDIASWITCH_IHBASE+0)	/* channel 0 DMA FIFO error */
#define IRQ_MSE_DFE1		(MEDIASWITCH_IHBASE+1)	/* channel 1 DMA FIFO error */
#define IRQ_MSE_DFE2		(MEDIASWITCH_IHBASE+2)	/* channel 2 DMA FIFO error */
#define IRQ_MSE_DFE3		(MEDIASWITCH_IHBASE+3)	/* channel 3 DMA FIFO error */
#define IRQ_MSE_DFE4		(MEDIASWITCH_IHBASE+4)	/* channel 4 DMA FIFO error */
#define IRQ_MSE_DFE5		(MEDIASWITCH_IHBASE+5)	/* channel 5 DMA FIFO error */
#define IRQ_MSE_DFE6		(MEDIASWITCH_IHBASE+6)	/* channel 6 DMA FIFO error */
#define IRQ_MSE_DFE7		(MEDIASWITCH_IHBASE+7)	/* channel 7 DMA FIFO error */
#define IRQ_MSE_I2C0_ACK	(MEDIASWITCH_IHBASE+8)	/* I2C 0 Acknowledge Error */
#define IRQ_MSE_I2C0_ARB	(MEDIASWITCH_IHBASE+9)	/* I2C 0 Arbitration Error */
#define IRQ_MSE_I2C1_ACK	(MEDIASWITCH_IHBASE+10)	/* I2C 1 Acknowledge Error */
#define IRQ_MSE_I2C1_ARB	(MEDIASWITCH_IHBASE+11)	/* I2C 1 Arbitration Error */
#define IRQ_MSE_PE_MAO		(MEDIASWITCH_IHBASE+12)	/* matching address overlap */
#define IRQ_MSE_PE_SM		(MEDIASWITCH_IHBASE+13)	/* parser state machine error */
#define IRQ_MSE_PE_MQO		(MEDIASWITCH_IHBASE+14)	/* mbus queue overflow */
#define IRQ_MSE_PE_MBEE		(MEDIASWITCH_IHBASE+15)	/* mbus byte enable error */

#define MEDIASWITCH_ITBASE	104

#define MEDIASWITCH_IGBASE	120

#define IRQ_MSG_GPIO0		(MEDIASWITCH_IGBASE+0)	/* GPIO0 interrupt */
#define IRQ_MSG_GPIO1		(MEDIASWITCH_IGBASE+1)	/* GPIO1 interrupt */
#define IRQ_MSG_GPIO2		(MEDIASWITCH_IGBASE+2)	/* GPIO2 interrupt */
#define IRQ_MSG_GPIO3		(MEDIASWITCH_IGBASE+3)	/* GPIO3 interrupt */
#define IRQ_MSG_GPIO4		(MEDIASWITCH_IGBASE+4)	/* GPIO4 interrupt */
#define IRQ_MSG_GPIO5		(MEDIASWITCH_IGBASE+5)	/* GPIO5 interrupt */
#define IRQ_MSG_GPIO6		(MEDIASWITCH_IGBASE+6)	/* GPIO6 interrupt */
#define IRQ_MSG_GPIO7		(MEDIASWITCH_IGBASE+7)	/* GPIO7 interrupt */
#define IRQ_MSG_GPIO8		(MEDIASWITCH_IGBASE+8)	/* GPIO8 interrupt */
#define IRQ_MSG_GPIO9		(MEDIASWITCH_IGBASE+9)	/* GPIO9 interrupt */
#define IRQ_MSG_GPIO10		(MEDIASWITCH_IGBASE+10)	/* GPIO10 interrupt */
#define IRQ_MSG_GPIO11		(MEDIASWITCH_IGBASE+11)	/* GPIO11 interrupt */
#define IRQ_MSG_GPIO12		(MEDIASWITCH_IGBASE+12)	/* GPIO12 interrupt */
#define IRQ_MSG_GPIO13		(MEDIASWITCH_IGBASE+13)	/* GPIO13 interrupt */
#define IRQ_MSG_GPIO14		(MEDIASWITCH_IGBASE+14)	/* GPIO14 interrupt */
#define IRQ_MSG_GPIO15		(MEDIASWITCH_IGBASE+15)	/* GPIO15 interrupt */

#endif // __PX_HW_H__
